@@ -1,4 +1,4 @@
 from .builders import build_update  # noqa
-from .diff import build_update_diff  # noqa
+from .diff import build_update_diff, select_attributes_for_set_and_remove  # noqa
 from .core import UpdateItem, DiffedUpdateItem  # noqa
 from .versioned import versioned_diffed_update_item, VersionedUpdateFailure  # noqa
